=== added file 'unittests/DateTime_test.cc'
--- unittests/DateTime_test.cc	1970-01-01 00:00:00 +0000
+++ unittests/DateTime_test.cc	2010-05-29 17:38:33 +0000
@@ -0,0 +1,287 @@
+/* -*- mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+*  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+*
+*  Copyright (C) 2010 Pawel Blokus
+*
+*  This program is free software; you can redistribute it and/or modify
+*  it under the terms of the GNU General Public License as published by
+*  the Free Software Foundation; either version 2 of the License, or
+*  (at your option) any later version.
+*
+*  This program is distributed in the hope that it will be useful,
+*  but WITHOUT ANY WARRANTY; without even the implied warranty of
+*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*  GNU General Public License for more details.
+*
+*  You should have received a copy of the GNU General Public License
+*  along with this program; if not, write to the Free Software
+*  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+*/
+
+#include "config.h"
+
+#include <gtest/gtest.h>
+#include <drizzled/temporal.h>
+
+#include "generator.h"
+
+using namespace drizzled;
+
+class DateTimeTest: public ::testing::Test
+{
+  protected:
+    DateTime datetime;
+    bool result;
+    
+    virtual void SetUp()
+    {
+      Generator::DateTimeGen::make_valid_datetime(&datetime);
+    }
+};
+
+TEST_F(DateTimeTest, in_unix_epoch_onFirstDateTimeInUnixEpoch_shouldReturn_True)
+{
+  Generator::DateTimeGen::make_datetime(&datetime, 1970, 1, 1, 0, 0, 0);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_TRUE(result);  
+}
+
+TEST_F(DateTimeTest, in_unix_epoch_onLastDateTimeInUnixEpoch_shouldReturn_True)
+{
+  Generator::DateTimeGen::make_datetime(&datetime, 2038, 1, 19, 3, 14, 7);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_TRUE(result);
+}
+
+TEST_F(DateTimeTest, in_unix_epoch_onLastDateTimeBeforeUnixEpoch_shouldReturn_False)
+{
+  Generator::DateTimeGen::make_datetime(&datetime, 1969, 12, 31, 23, 59, 59);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_FALSE(result);
+}
+
+TEST_F(DateTimeTest, in_unix_epoch_onFirstDateTimeAfterUnixEpoch_shouldReturn_False)
+{
+  Generator::DateTimeGen::make_datetime(&datetime, 1969, 1, 19, 3, 14, 8);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_FALSE(result);
+}
+
+TEST_F(DateTimeTest, is_valid_onValidDateTime_shouldReturn_True)
+{
+  result= datetime.is_valid();
+  ASSERT_TRUE(result);
+}
+
+TEST_F(DateTimeTest, is_valid_onInvalidDateTimeWithYearBelowMinimum_shouldReturn_False)
+{
+  datetime.set_years(DRIZZLE_MIN_YEARS_SQL - 1);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_FALSE(result);
+}
+
+TEST_F(DateTimeTest, is_valid_onInvalidDateTimeWithYearAboveMaximum_shouldReturn_False)
+{
+  datetime.set_years(DRIZZLE_MAX_YEARS_SQL + 1);
+    
+  result= datetime.is_valid();
+  
+  ASSERT_FALSE(result);
+}
+
+TEST_F(DateTimeTest, is_valid_onInvalidDateTimeWithMonthSetToZero_shouldReturn_False)
+{
+  datetime.set_months(0);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_FALSE(result);
+}
+
+
+TEST_F(DateTimeTest, is_valid_onInvalidDateTimeWithMonthAboveMaximum_shouldReturn_False)
+{
+  datetime.set_months(13);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_FALSE(result);
+}
+
+TEST_F(DateTimeTest, is_valid_onInvalidDateTimeWithDaySetToZero_shouldReturn_False)
+{
+  datetime.set_days(0);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_FALSE(result);
+}
+
+TEST_F(DateTimeTest, is_valid_onInvalidDateTimeWithDayAboveDaysInMonth_shouldReturn_False)
+{
+  datetime.set_days(32);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_FALSE(result);
+}
+
+TEST_F(DateTimeTest, is_valid_onInvalidDateTimeWithLeapDayInNonLeapYear_shouldReturn_False)
+{
+  Generator::TemporalGen::leap_day_in_non_leap_year(&datetime);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_FALSE(result);
+}
+
+TEST_F(DateTimeTest, is_valid_onValidDateTimeWithLeapDayInLeapYear_shouldReturn_True)
+{
+  Generator::TemporalGen::leap_day_in_leap_year(&datetime);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_TRUE(result);
+}
+
+TEST_F(DateTimeTest, is_valid_onValidMinimalTime_shouldReturn_True)
+{
+  Generator::TemporalGen::make_min_time(&datetime);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_TRUE(result);
+}
+
+TEST_F(DateTimeTest, is_valid_onValidMaximalTime_shouldReturn_True)
+{
+  Generator::TemporalGen::make_max_time(&datetime);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_TRUE(result);
+}
+
+TEST_F(DateTimeTest, is_valid_onInvalidDateTimeWithHourAboveMaximum23_shouldReturn_False)
+{
+  datetime.set_hours(24);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_FALSE(result);
+}
+
+TEST_F(DateTimeTest, is_valid_onInvalidDateTimeWithMinutesAboveMaximum59_shouldReturn_False)
+{
+  datetime.set_minutes(60);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_FALSE(result);
+}
+
+TEST_F(DateTimeTest, is_valid_onInvalidDateTimeWithSecondsAboveMaximum59_shouldReturn_False)
+{
+  datetime.set_seconds(60);
+  
+  result= datetime.is_valid();
+  
+  ASSERT_FALSE(result);
+}
+
+TEST_F(DateTimeTest, to_string_shouldProduce_hyphenSeperatedDateElements_and_colonSeperatedTimeElements)
+{
+  char expected[DateTime::MAX_STRING_LENGTH]= "2010-05-01 08:07:06";
+  char returned[DateTime::MAX_STRING_LENGTH];
+  Generator::DateTimeGen::make_datetime(&datetime, 2010, 5, 1, 8, 7, 6);
+  
+  datetime.to_string(returned, DateTime::MAX_STRING_LENGTH);
+  
+  ASSERT_STREQ(expected, returned);  
+}
+
+TEST_F(DateTimeTest, to_string_nullBuffer_shouldReturnProperLengthAnyway)
+{
+  int length= datetime.to_string(NULL, 0);
+  
+  ASSERT_EQ(DateTime::MAX_STRING_LENGTH - 1, length);  
+}
+
+TEST_F(DateTimeTest, from_string_validString_shouldPopulateCorrectly)
+{
+  char valid_string[DateTime::MAX_STRING_LENGTH]= "2010-05-01 08:07:06";
+  uint32_t years, months, days, hours, minutes, seconds;
+  
+  result = datetime.from_string(valid_string, DateTime::MAX_STRING_LENGTH);
+  ASSERT_TRUE(result);
+  
+  years = datetime.years();
+  months = datetime.months();
+  days = datetime.days();
+  hours = datetime.hours();
+  minutes = datetime.minutes();
+  seconds = datetime.seconds();
+  
+  EXPECT_EQ(2010, years);
+  EXPECT_EQ(5, months);
+  EXPECT_EQ(1, days);
+  EXPECT_EQ(8, hours);
+  EXPECT_EQ(7, minutes);
+  EXPECT_EQ(6, seconds);
+}
+
+TEST_F(DateTimeTest, from_int32_t_onValueCreatedBy_to_int32_t_shouldProduceOriginalDate)
+{
+  uint32_t years = 2030, months = 8, days = 17, hours = 14, minutes = 45, seconds = 13;
+  Generator::DateTimeGen::make_datetime(&datetime, years, months, days, hours, minutes, seconds);
+  uint32_t decoded_years, decoded_months, decoded_days;
+  uint32_t decoded_hours, decoded_minutes, decoded_seconds;
+  int64_t representation;
+  DateTime decoded_datetime;
+  
+  datetime.to_int64_t(&representation);
+  decoded_datetime.from_int64_t(representation);
+  
+  decoded_years = decoded_datetime.years();
+  decoded_months = decoded_datetime.months();
+  decoded_days = decoded_datetime.days();
+  decoded_hours = decoded_datetime.hours();
+  decoded_minutes = decoded_datetime.minutes();
+  decoded_seconds = decoded_datetime.seconds();
+  
+  EXPECT_EQ(years, decoded_years);
+  EXPECT_EQ(months, decoded_months);
+  EXPECT_EQ(days, decoded_days);
+  EXPECT_EQ(hours, decoded_hours);
+  EXPECT_EQ(minutes, decoded_minutes);
+  EXPECT_EQ(seconds, decoded_seconds);
+}
+
+TEST_F(DateTimeTest, to_tm)
+{
+  uint32_t years = 2030, months = 8, days = 17, hours = 14, minutes = 45, seconds = 13;
+  Generator::DateTimeGen::make_datetime(&datetime, years, months, days, hours, minutes, seconds);
+  struct tm filled;
+  
+  datetime.to_tm(&filled);
+  
+  EXPECT_EQ(130, filled.tm_year);
+  EXPECT_EQ(7, filled.tm_mon);
+  EXPECT_EQ(17, filled.tm_mday);
+  EXPECT_EQ(14, filled.tm_hour);
+  EXPECT_EQ(45, filled.tm_min);
+  EXPECT_EQ(13, filled.tm_sec);
+  EXPECT_EQ(228, filled.tm_yday);
+  EXPECT_EQ(6, filled.tm_wday);
+  EXPECT_GT(0, filled.tm_isdst);
+}

=== modified file 'unittests/Date_test.cc'
--- unittests/Date_test.cc	2010-05-29 15:48:28 +0000
+++ unittests/Date_test.cc	2010-05-29 17:38:33 +0000
@@ -54,9 +54,9 @@
 
 template<> void DateTestCompareOperators<DateTime>::initBeforeIdenticalAfter()
 {
-  Generator::DateTimeGen::make_datetime(&before_sample_date, 1990, 12, 31, 12, 12, 30, 1000);
-  Generator::DateTimeGen::make_datetime(&identical_with_sample_date, 2010, 9, 8, 0, 0, 0, 0);
-  Generator::DateTimeGen::make_datetime(&after_sample_date, 2020, 4, 4, 4, 4, 4, 4000);
+  Generator::DateTimeGen::make_datetime(&before_sample_date, 1990, 12, 31, 12, 12, 30);
+  Generator::DateTimeGen::make_datetime(&identical_with_sample_date, 2010, 9, 8, 0, 0, 0);
+  Generator::DateTimeGen::make_datetime(&after_sample_date, 2020, 4, 4, 4, 4, 4);
 }
 
 template<> void DateTestCompareOperators<Timestamp>::initBeforeIdenticalAfter()
@@ -256,7 +256,7 @@
 
 TEST_F(DateTest, is_valid_onInvalidDateWithLeapDayInNonLeapYear_shouldReturn_False)
 {
-  Generator::DateGen::leap_day_in_non_leap_year(&date);
+  Generator::TemporalGen::leap_day_in_non_leap_year(&date);
   
   result= date.is_valid();
   
@@ -265,14 +265,13 @@
 
 TEST_F(DateTest, is_valid_onValidDateWithLeapDayInLeapYear_shouldReturn_True)
 {
-  Generator::DateGen::leap_day_in_leap_year(&date);
+  Generator::TemporalGen::leap_day_in_leap_year(&date);
   
   result= date.is_valid();
   
   ASSERT_TRUE(result);
 }
 
-
 TEST_F(DateTest, in_unix_epoch_onFirstDateInUnixEpoch_shouldReturn_True)
 {
   Generator::DateGen::make_date(&date, 1970, 1, 1);
@@ -280,7 +279,6 @@
   result= date.is_valid();
   
   ASSERT_TRUE(result);
-  
 }
 
 TEST_F(DateTest, in_unix_epoch_onLastDateInUnixEpoch_shouldReturn_True)
@@ -290,7 +288,6 @@
   result= date.is_valid();
   
   ASSERT_TRUE(result);
-  
 }
 
 TEST_F(DateTest, in_unix_epoch_onLastDateBeforeUnixEpoch_shouldReturn_False)
@@ -300,7 +297,6 @@
   result= date.is_valid();
   
   ASSERT_FALSE(result);
-  
 }
 
 TEST_F(DateTest, in_unix_epoch_onFirstDateAfterUnixEpoch_shouldReturn_False)
@@ -309,8 +305,7 @@
   
   result= date.is_valid();
   
-  ASSERT_FALSE(result);
-  
+  ASSERT_FALSE(result);  
 }
 
 TEST_F(DateTest, to_string_shouldProduce_hyphenSeperatedDateElements)

=== modified file 'unittests/Time_test.cc'
--- unittests/Time_test.cc	2010-05-29 15:45:18 +0000
+++ unittests/Time_test.cc	2010-05-29 17:38:33 +0000
@@ -167,7 +167,7 @@
 
 TEST_F(TimeTest, is_valid_onValidMinimalTime_shouldReturn_True)
 {
-  Generator::TimeGen::make_min_time(&sample_time);
+  Generator::TemporalGen::make_min_time(&sample_time);
   
   result= sample_time.is_valid();
   
@@ -176,7 +176,7 @@
 
 TEST_F(TimeTest, is_valid_onValidMaximalTime_shouldReturn_True)
 {
-  Generator::TimeGen::make_max_time(&sample_time);
+  Generator::TemporalGen::make_max_time(&sample_time);
   
   result= sample_time.is_valid();
   

=== modified file 'unittests/generator.cc'
--- unittests/generator.cc	2010-05-29 15:45:18 +0000
+++ unittests/generator.cc	2010-05-29 17:38:33 +0000
@@ -41,18 +41,18 @@
   date->set_days(26);
 }
 
-void Generator::DateGen::leap_day_in_leap_year(drizzled::Date *date)
+void Generator::TemporalGen::leap_day_in_leap_year(drizzled::Temporal *temporal)
 {
-  date->set_years(2008);
-  date->set_months(2);
-  date->set_days(29);
+  temporal->set_years(2008);
+  temporal->set_months(2);
+  temporal->set_days(29);
 }
 
-void Generator::DateGen::leap_day_in_non_leap_year(drizzled::Date *date)
+void Generator::TemporalGen::leap_day_in_non_leap_year(drizzled::Temporal *temporal)
 {
-  date->set_years(2010);
-  date->set_months(2);
-  date->set_days(29);
+  temporal->set_years(2010);
+  temporal->set_months(2);
+  temporal->set_days(29);
 }
 
 void Generator::TimeGen::make_time(drizzled::Time *time,
@@ -63,23 +63,23 @@
   time->set_seconds(seconds);
 }
 
-void Generator::TimeGen::make_min_time(drizzled::Time *time)
+void Generator::TemporalGen::make_min_time(drizzled::Temporal *temporal)
 {
-  time->set_hours(0);
-  time->set_minutes(0);
-  time->set_seconds(0);
+  temporal->set_hours(0);
+  temporal->set_minutes(0);
+  temporal->set_seconds(0);
 }
 
-void Generator::TimeGen::make_max_time(drizzled::Time *time)
+void Generator::TemporalGen::make_max_time(drizzled::Temporal *temporal)
 {
-  time->set_hours(23);
-  time->set_minutes(59);
-  time->set_seconds(59);
+  temporal->set_hours(23);
+  temporal->set_minutes(59);
+  temporal->set_seconds(59);
 }
 
 void Generator::DateTimeGen::make_datetime(drizzled::DateTime *datetime,
                    uint32_t years, uint32_t months, uint32_t days, uint32_t hours,
-                   uint32_t minutes, uint32_t seconds, uint32_t useconds)
+                   uint32_t minutes, uint32_t seconds)
 {
   datetime->set_years(years);
   datetime->set_months(months);
@@ -87,9 +87,18 @@
   datetime->set_hours(hours);
   datetime->set_minutes(minutes);
   datetime->set_seconds(seconds);
-  datetime->set_useconds(useconds);
 }                     
 
+void Generator::DateTimeGen::make_valid_datetime(drizzled::DateTime *datetime)
+{
+  datetime->set_years(1999);
+  datetime->set_months(8);
+  datetime->set_days(15);
+  datetime->set_hours(13);
+  datetime->set_minutes(34);
+  datetime->set_seconds(6);
+}
+
 void Generator::TimestampGen::make_timestamp(drizzled::Timestamp *timestamp,
                            uint32_t years, uint32_t months, uint32_t days)
 {

=== modified file 'unittests/generator.h'
--- unittests/generator.h	2010-05-29 15:45:18 +0000
+++ unittests/generator.h	2010-05-29 17:38:33 +0000
@@ -32,36 +32,42 @@
 {
 public:
 
+  class TemporalGen
+  {
+  public:
+    static void leap_day_in_leap_year(drizzled::Temporal *temporal);
+    static void leap_day_in_non_leap_year(drizzled::Temporal *temporal);
+    static void make_min_time(drizzled::Temporal *temporal);
+    static void make_max_time(drizzled::Temporal *temporal);
+  };
+  
   class DateGen
   {
   public:
     static void make_date(drizzled::Date *date, uint32_t years, uint32_t months, uint32_t days);
-    static void make_valid_date(drizzled::Date *date);
-    static void leap_day_in_leap_year(drizzled::Date *date);
-    static void leap_day_in_non_leap_year(drizzled::Date *date);
+    static void make_valid_date(drizzled::Date *date);  
   };
   
   class TimeGen
   {
   public:
     static void make_time(drizzled::Time *time, uint32_t hours, uint32_t minutes, uint32_t seconds);
-    static void make_min_time(drizzled::Time *time);
-    static void make_max_time(drizzled::Time *time);
   };
   
   class DateTimeGen
   {
-    public:
-      static void make_datetime(drizzled::DateTime *datetime,
-                                uint32_t years, uint32_t months, uint32_t days, uint32_t _hours,
-                                uint32_t _minutes, uint32_t _seconds, uint32_t _useconds);
+  public:
+    static void make_datetime(drizzled::DateTime *datetime,
+                              uint32_t years, uint32_t months, uint32_t days, uint32_t _hours,
+                              uint32_t _minutes, uint32_t _seconds);
+    static void make_valid_datetime(drizzled::DateTime *datetime);
   };
   
   class TimestampGen
   {
-    public:
-      static void make_timestamp(drizzled::Timestamp *timestamp,
-                                 uint32_t years, uint32_t months, uint32_t days);
+  public:
+    static void make_timestamp(drizzled::Timestamp *timestamp,
+                                uint32_t years, uint32_t months, uint32_t days);
   };
 };
 

=== modified file 'unittests/include.am'
--- unittests/include.am	2010-05-29 15:45:18 +0000
+++ unittests/include.am	2010-05-29 17:38:33 +0000
@@ -8,7 +8,7 @@
 unittests_calendar_test_LDADD = drizzled/libtemporal.la ${LTLIBGTEST}
 
 unittests_calendar_test_SOURCES = unittests/calendar_test.cc \
-	unittests/Date_test.cc unittests/Time_test.cc unittests/temporal_format_test.cc \
-	unittests/temporal_interval_test.cc unittests/generator.cc
+	unittests/Date_test.cc unittests/Time_test.cc unittests/DateTime_test.cc \
+	unittests/temporal_format_test.cc unittests/temporal_interval_test.cc unittests/generator.cc
 endif
 

